@@ -1,6 +1,8 @@
 from copy import deepcopy
 from enum import IntEnum
+from pathlib import Path
 from typing import Any, Dict, List
+from urllib.parse import unquote
 from xml.etree import ElementTree as ET
 
 from nonebot.typing import overrides
@@ -102,6 +104,10 @@ class MessageEvent(Event):
     def get_type(self) -> str:
         return "message"
 
+    @overrides(Event)
+    def get_user_id(self) -> str:
+        return self.from_wxid
+
     @overrides(Event)
     def get_message(self) -> "Message":
         return self.message
@@ -143,10 +149,10 @@ class QuoteMessageEvent(MessageEvent):
     def get_pre_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         raw_xml = values["raw_msg"]
         xml_obj = ET.fromstring(raw_xml)
-        values["message"] = xml_obj.findall("./appmsg/title")[0].text
-        refermsg = xml_obj.findall("./appmsg/refermsg")[0]
-        values["quote_message_id"] = refermsg.findall("./svrid")[0].text
-        values["quote_uer_id"] = refermsg.findall("./chatusr")[0].text
+        values["message"] = xml_obj.find("./appmsg/title").text
+        refermsg = xml_obj.find("./appmsg/refermsg")
+        values["quote_message_id"] = refermsg.find("./svrid").text
+        values["quote_uer_id"] = refermsg.find("./chatusr").text
         return values
 
     @overrides(MessageEvent)
@@ -178,6 +184,8 @@ class VoiceMessageEvent(MessageEvent):
     """接收语音消息"""
 
     type: int = EventType.MT_RECV_VOICE_MSG
+    mp3_file: str
+    """语音消息文件"""
     raw_msg: str
     """微信中的原始消息,xml格式"""
 
@@ -194,22 +202,38 @@ class CardMessageEvent(MessageEvent):
     """接收名片消息"""
 
     type: int = EventType.MT_RECV_CARD_MSG
+    headimg_url: str
+    """头像url"""
+    nickname: str
+    """名片用户昵称"""
     raw_msg: str
     """微信中的原始消息,xml格式"""
 
+    @root_validator(pre=True, allow_reuse=True)
+    def get_pre_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        raw_xml = values["raw_msg"]
+        xml_obj = ET.fromstring(raw_xml)
+        values["headimg_url"] = xml_obj.attrib.get("bigheadimgurl")
+        values["nickname"] = xml_obj.attrib.get("nickname")
+        return values
+
     @overrides(MessageEvent)
     def get_event_description(self) -> str:
-        msg = "[名片消息]请查看raw_msg"
+        msg = f"[名片消息] - {self.nickname}"
         if self.room_wxid:
             return f"Message {self.msgid} from {self.from_wxid}@[群:{self.room_wxid}]: {msg}"
         else:
             return f"Message {self.msgid} from {self.from_wxid}: {msg}"
 
 
-class ViedeoMessageEvent(MessageEvent):
+class VideoMessageEvent(MessageEvent):
     """接收视频消息"""
 
     type: int = EventType.MT_RECV_VIDEO_MSG
+    video: str
+    "接收视频路径"
+    video_thumb: str
+    "视频缩略图路径"
     raw_msg: str
     """微信中的原始消息,xml格式"""
 
@@ -226,9 +250,19 @@ class EmojiMessageEvent(MessageEvent):
     """接收表情消息"""
 
     type = EventType.MT_RECV_EMOJI_MSG
+    emoji_url: str
+    """表情url地址"""
     raw_msg: str
     """微信中的原始消息,xml格式"""
 
+    @root_validator(pre=True, allow_reuse=True)
+    def get_pre_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        raw_xml = values["raw_msg"]
+        xml_obj = ET.fromstring(raw_xml)
+        emoji_url = xml_obj.find("./emoji").attrib.get("cdnurl")
+        values["emoji_url"] = unquote(emoji_url)
+        return values
+
     @overrides(MessageEvent)
     def get_event_description(self) -> str:
         msg = "[表情消息]请查看raw_msg"
@@ -242,12 +276,58 @@ class LocationMessageEvent(MessageEvent):
     """接收位置消息消息"""
 
     type: int = EventType.MT_RECV_LOCATION_MSG
+    location_x: str
+    """位置x坐标"""
+    location_y: str
+    """位置y坐标"""
+    label: str
+    """位置标签"""
+    poiname: str
+    """位置名称"""
     raw_msg: str
     """微信中的原始消息,xml格式"""
 
+    @root_validator(pre=True, allow_reuse=True)
+    def get_pre_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        raw_xml = values["raw_msg"]
+        xml_obj = ET.fromstring(raw_xml)
+        location = xml_obj.find("./location")
+        values["location_x"] = location.attrib.get("x")
+        values["location_y"] = location.attrib.get("y")
+        values["label"] = location.attrib.get("label")
+        values["poiname"] = location.attrib.get("poiname")
+        return values
+
     @overrides(MessageEvent)
     def get_event_description(self) -> str:
-        msg = "[位置消息]请查看raw_msg"
+        msg = f"[位置消息] - {self.poiname}"
+        if self.room_wxid:
+            return f"Message {self.msgid} from {self.from_wxid}@[群:{self.room_wxid}]: {msg}"
+        else:
+            return f"Message {self.msgid} from {self.from_wxid}: {msg}"
+
+
+class FileMessageEvent(MessageEvent):
+    """接收文件消息"""
+
+    type: int = EventType.MT_RECV_FILE_MSG
+    wx_sub_type: int = SubType.WX_APPMSG_FILE
+    file: str
+    """接收文件路径"""
+    file_name: str
+    """接收文件名称"""
+    raw_msg: str
+    """微信中的原始消息,xml格式"""
+
+    @root_validator(pre=True, allow_reuse=True)
+    def get_pre_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        file = Path(values["file"])
+        values["file_name"] = file.name
+        return values
+
+    @overrides(Event)
+    def get_event_description(self) -> str:
+        msg = f"[接收文件事件] - {self.file_name}"
         if self.room_wxid:
             return f"Message {self.msgid} from {self.from_wxid}@[群:{self.room_wxid}]: {msg}"
         else:
@@ -277,6 +357,10 @@ class SystemMessageEvent(Event):
     def get_type(self) -> str:
         return "system"
 
+    @overrides(Event)
+    def get_user_id(self) -> str:
+        return self.from_wxid
+
     @overrides(Event)
     def get_event_description(self) -> str:
         msg = "[系统消息]请查看raw_msg"
@@ -309,6 +393,10 @@ class OtherMessageEvent(Event):
     def get_type(self) -> str:
         return "other"
 
+    @overrides(Event)
+    def get_user_id(self) -> str:
+        return self.from_wxid
+
     @overrides(Event)
     def get_event_description(self) -> str:
         msg = "[其他未知消息]请查看对应type和raw_msg"
@@ -338,6 +426,10 @@ class RequestEvent(Event):
     def get_type(self) -> str:
         return "request"
 
+    @overrides(Event)
+    def get_user_id(self) -> str:
+        return self.from_wxid
+
 
 class FriendAddRequestEvent(RequestEvent):
     """添加好友请求"""
@@ -380,11 +472,15 @@ class RevokeNoticeEvent(NoticeEvent):
     msg_id: str
     """撤回消息id"""
 
+    @overrides(NoticeEvent)
+    def get_user_id(self) -> str:
+        return self.from_wxid
+
     @root_validator(pre=True, allow_reuse=True)
     def get_pre_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         raw_xml = values["raw_msg"]
         xml_obj = ET.fromstring(raw_xml)
-        values["msg_id"] = xml_obj.findall("./revokemsg/newmsgid")[0].text
+        values["msg_id"] = xml_obj.find("./revokemsg/newmsgid").text
         return values
 
     @overrides(NoticeEvent)
@@ -492,6 +588,10 @@ class AppEvent(Event):
     def get_type(self) -> str:
         return "app"
 
+    @overrides(Event)
+    def get_user_id(self) -> str:
+        return self.from_wxid
+
     @overrides(Event)
     def get_event_name(self) -> str:
         wx_type = WxType(self.wx_type).name
@@ -519,23 +619,6 @@ class LinkMessageEvent(AppEvent):
             return f"Message {self.msgid} from {self.from_wxid}: {msg}"
 
 
-class FileMessageEvent(AppEvent):
-    """接收文件消息"""
-
-    type: int = EventType.MT_RECV_FILE_MSG
-    wx_sub_type: int = SubType.WX_APPMSG_FILE
-    raw_msg: str
-    """微信中的原始消息,xml格式"""
-
-    @overrides(Event)
-    def get_event_description(self) -> str:
-        msg = "[接收文件事件]请查看raw_msg"
-        if self.room_wxid:
-            return f"Message {self.msgid} from {self.from_wxid}@[群:{self.room_wxid}]: {msg}"
-        else:
-            return f"Message {self.msgid} from {self.from_wxid}: {msg}"
-
-
 class MiniAppMessageEvent(AppEvent):
     """接收小程序消息"""
 
@@ -558,12 +641,25 @@ class WcpayMessageEvent(AppEvent):
 
     type: int = EventType.MT_RECV_WCPAY_MSG
     wx_sub_type: int = SubType.WX_APPMSG_WCPAY
+    feedesc: str
+    """收钱数目，以￥开头"""
+    pay_memo: str
+    """转账说明"""
     raw_msg: str
     """微信中的原始消息,xml格式"""
 
+    @root_validator(pre=True, allow_reuse=True)
+    def get_pre_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        raw_xml = values["raw_msg"]
+        xml_obj = ET.fromstring(raw_xml)
+        wcpayinfo = xml_obj.find("./appmsg/wcpayinfo")
+        values["feedesc"] = wcpayinfo.find("./feedesc").text
+        values["pay_memo"] = wcpayinfo.find("./pay_memo").text
+        return values
+
     @overrides(Event)
     def get_event_description(self) -> str:
-        msg = "[转账消息]请查看raw_msg"
+        msg = f"[转账消息] - 收到{self.feedesc}，转账说明：{self.pay_memo}"
         if self.room_wxid:
             return f"Message {self.msgid} from {self.from_wxid}@[群:{self.room_wxid}]: {msg}"
         else:
